package cn.yyxx.commsdk.core.impl;

import android.app.Activity;
import android.app.Application;
import android.content.Context;
import android.content.Intent;
import android.content.res.Configuration;
import android.os.Bundle;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.List;
import java.util.Map;

import cn.yyxx.commsdk.base.entity.Function;
import cn.yyxx.commsdk.base.entity.ResultInfo;
import cn.yyxx.commsdk.base.entity.SdkChargeInfo;
import cn.yyxx.commsdk.base.entity.SdkRoleInfo;
import cn.yyxx.commsdk.base.internal.ICallback;

import cn.yyxx.commsdk.base.utils.Logger;

import cn.yyxx.commsdk.core.common.CommonOperationManager;

import cn.yyxx.commsdk.core.utils.MMKVUtils;
import cn.yyxx.support.permission.IPermissionCallback;
import cn.yyxx.support.permission.PermissionKit;


/**
 * @author #Suyghur.
 * Created on 2021/12/25
 */


public final class SdkBridgeImpl {

    private Object mSdkProxy;
    public SdkBridgeImpl(Context context) {

    }

    private Object getSdkProxyManager(Context context){
        if (mSdkProxy == null) {
            mSdkProxy = SdkProxyManager.initSdkProxyManager(context);
        }
        return mSdkProxy;
    }

    public void attachBaseContext(Application application, Context context) {
        Logger.i("application   attachBaseContext ...");
        // 初始化Zap日志框架
        Logger.initZap(application);
        CommonOperationManager.getInstance().initMsaDeviceIds(application);

        if (mSdkProxy == null) {
            mSdkProxy = getSdkProxyManager(application);
        }
        SdkProxyManager.call(Function.ATTACH_BASE_CONTEXT,  new Class[]{Application.class, Context.class}, new Object[]{application, context});


    }

    public void initApplication(Application application) {
        Logger.i("initApplication ...");
        MMKVUtils.getInstance().init(application);
        if (mSdkProxy == null) {
            mSdkProxy = getSdkProxyManager(application);
        }
        SdkProxyManager.call(Function.INIT_APPLICATION,  new Class[]{Application.class}, new Object[]{application});

    }

    public void initAgreement(Activity activity,ICallback callback) {

        CommonOperationManager.getInstance().initAgreement(activity,callback);
    }
    public void initialize(Activity activity, ICallback callback) {


        if (mSdkProxy == null) {
            mSdkProxy = getSdkProxyManager(activity);
        }

        SdkProxyManager.call(Function.INITIALIZE,  new Class[]{Activity.class, ICallback.class}, new Object[]{activity,callback});

        CommonOperationManager.getInstance().initialize(activity, new ICallback() {
            @Override
            public void onResult(int code, String result) {
                callback.onResult(code,result);
                if (code == ResultInfo.CODE_SUCCESS){
                    SdkProxyManager.call(Function.INITIALIZE_SUCCESS,  new Class[]{}, new Object[]{});
                }
            }
        });

    }

    public void setLogoutListener(Activity activity,ICallback callback){
        if (mSdkProxy == null) {
            mSdkProxy = getSdkProxyManager(activity);
        }

        SdkProxyManager.call(Function.SET_LOGOUT_LISTENER,  new Class[]{Activity.class, ICallback.class}, new Object[]{activity,callback});

    }

    public void login(Activity activity, final ICallback callback) {
        if (mSdkProxy == null) {
            mSdkProxy = getSdkProxyManager(activity);
        }

        SdkProxyManager.call(Function.LOGIN,  new Class[]{Activity.class, ICallback.class}, new Object[]{activity,callback});

    }

    public void logout() {
        SdkProxyManager.call(Function.LOGOUT,  new Class[]{}, new Object[]{});

    }

    public void charge(Activity activity, SdkChargeInfo chargeInfo, ICallback callback) {
    }

    public void roleCreate(Activity activity, SdkRoleInfo roleInfo, ICallback callback) {
    }

    public void roleLauncher(Activity activity, SdkRoleInfo roleInfo, ICallback callback) {
    }

    public void roleUpgrade(Activity activity, SdkRoleInfo roleInfo, ICallback callback) {
    }

    public boolean hasExitView(Activity activity) {
        return false;
    }

    public void openExitView(Activity activity, final ICallback callback) {
    }

    public void onStart(Activity activity) {
    }

    public void onRestart(Activity activity) {
    }

    public void onResume(Activity activity) {
    }


    public void onPause(Activity activity) {
    }

    public void onStop(Activity activity) {
    }

    public void onDestroy(Activity activity) {
    }

    public void onActivityResult(Activity activity, int requestCode, int resultCode, Intent intent) {
    }

    public void onNewIntent(Activity activity, Intent intent) {
    }

    public void onConfigurationChanged(Activity activity, Configuration newConfigs){
    }
    public void onCreate(Activity activity, Bundle saveInstanceState){

    }
    public void requestPermission(Activity activity, List<String> permissions, ICallback callback) {
        Logger.i(" request  extend permission ...");
        PermissionKit.with().permission(permissions).request(activity, new IPermissionCallback() {
            @Override
            public void onGranted(List<String> list, boolean all) {
                Logger.i(" request  extend permission:onGranted");
                JSONObject jsonObject = permissionList2Json(list);
                if (jsonObject != null) {
                    callback.onResult(0, jsonObject.toString());
                } else {
                    callback.onResult(0, "");
                }
            }

            @Override
            public void onDenied(List<String> list, boolean never) {

                Logger.i(" request  extend permission :onDenied...");


                JSONObject jsonObject = permissionList2Json(list);
                if (jsonObject != null) {
                    callback.onResult(-1, jsonObject.toString());
                } else {
                    callback.onResult(-1, "");
                }
            }

            @Override
            public void onProxyFinish() {
                Logger.i(" request  extend permission :onProxyFinish...");

            }
        });

    }



    public String getCurrentUserId() {
        return "";
    }


    private JSONObject permissionList2Json(List<String> list) {
        try {
            JSONObject jsonObject = new JSONObject();
            JSONArray jsonArray = new JSONArray();
            for (String permission : list) {
                jsonArray.put(permission);
            }
            jsonObject.put("permissions", jsonArray);
            return jsonObject;
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return null;
    }

}
